/*
 * This file is part of leds.
 * 
 * leds is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * leds is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with leds; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

module leds.BrowserView;

private import leds.Leds;
private import leds.Directory;
private import leds.Workspace;
private import leds.FileView;
private import leds.Docker;
private import leds.Project;
//private import leds.POpen;
private import leds.ExecConsole;

private import dui.All;
private import std.string;
private import std.path;
private import std.conv;

private import parse.Module;

private class RowType
{
	static const char[] MODULE = "M";
	static const char[] IMPORT = "I";
	static const char[] ELEMENT = "E";
}

/**
 * Displays the content of a Module in a TreeView
 */
public:
class BrowserView : VBox , ButtonClickedListener , Dockable
{
	
	Workspace workspace;

	char[] currentDir;

	BrowserTreeView treeView;
	ScrolledWindow sw;

	// our tree definition
	/**
	 * the columns:<br>
	 * 0 = visible to user
	 * 1 = type of information
	 * 2 = data (depending on type of information) for elements the line number as a char[]
	 */
	static GType [3] columns = [DUIType.STRING,DUIType.STRING,DUIType.STRING]; 
	
	/** the tree store */
	BrowserTreeStore browserTreeStore;
	/** the root iter */
	TreeIter iterTop;
	/** the current projects */
	//Project [char[]] projects;

	Menu tabMenu;

	Docker docker;

	Module sourceModule;

	/**
	 * Creates a new project view for a specific workspace
	 * @param workspace the workspace
	 */
	this(Workspace workspace)
	{
		super(false,0);
		this.workspace = workspace;
		
		currentDir = ".";
		
		sw = new ScrolledWindow(PolicyType.AUTOMATIC, PolicyType.AUTOMATIC);
		treeView = createBrowserTree();
		sw.add(treeView);
		packStart(getToolbar(),false,false,0);
		packStart(sw,true,true,0);
		
	}

	Widget getWidget()
	{
		return this;
	}
	
	char[] getTabName()
	{
		return "br";
	}
	
	Menu getTabMenu()
	{
		if ( tabMenu === null )
		{
			tabMenu = new Menu();
			MenuItem item;
			
			//tabMenu.append(new MenuItem(this,"_Close","tabMenu.close"));
			//tabMenu.append(new MenuItem(this,"_Float","tabMenu.float"));
			tabMenu.append(new MenuItem("test browser"));
		}
		//return tabMenu;
		return null;
	}

	void setDocker(Docker docker)
	{
		this.docker = docker;
	}

	/**
	 * Gets this dockable file view
	 * @return null because with don't have a file view
	 */
	FileView getFileView()
	{
		return null;
	}
	
	/**
	 * detrimines if a close button shoud be visible
	 */
	bit allowCloseButton()
	{
		return false;
	}
	
	bit allowClose()
	{
		return true;
	}

	bit clear()
	{
		return false;
	}

	bit parse(char[] text)
	{
	printf("BrowserView.parse %d\n",text.length);
		sourceModule = new Module("",text, Access.NONE);
		sourceModule.print();
		displayModule(sourceModule);
		return false;
	}

	void displayModule(Module sourceModule)
	{
		TreeIter oldIter = iterTop;
		iterTop = browserTreeStore.createIter();

		static int [3] cols = [0,1,2];
		char [][] subVals;
		subVals ~= sourceModule.getModuleName();
		subVals ~= RowType.MODULE;
		subVals ~= std.string.toString(sourceModule.getLineInCode());
		browserTreeStore.set(iterTop,cols,subVals);

		addImports(iterTop, sourceModule.imports);
		addModuleElements(iterTop, sourceModule.elements);
		addModuleFunctions(iterTop, sourceModule.functions);
		addModuleClasses(iterTop, sourceModule.classes);

		if ( oldIter !== null )
		{
			browserTreeStore.remove(oldIter);
		}
	}


	private void addImports(TreeIter parentIter, Element[] elements)
	{
		static int [3] cols = [0,1,2];
		if ( elements.length == 0 ) return;
		char[][] importHeader;
		importHeader ~= "imports";
		importHeader ~= RowType.IMPORT;
		importHeader ~= std.string.toString(0);
		parentIter = browserTreeStore.append(parentIter);
		browserTreeStore.set(parentIter,cols,importHeader);
		
		try
		{
			foreach(Element element ; elements)
			{
			//printf("adding element %.*s\n",element.returnType~" "~element.id);
				TreeIter iterChild = browserTreeStore.append(parentIter);
				char [][] subVals;
				subVals ~= element.getLongForm();
				subVals ~= RowType.ELEMENT;
				subVals ~= std.string.toString(element.getLineInCode());
				browserTreeStore.set(iterChild,cols,subVals);
			}
		}
		catch ( DirectoryError e)
		{
			printf("BrowserView addModuleElements error : %.*s\n",e.toString());
		}
	}

	private void addModuleElements(TreeIter parentIter, Element[] elements)
	{
		static int [3] cols = [0,1,2];
		if ( elements.length == 0 ) return;
		try
		{
			foreach(Element element ; elements)
			{
			//printf("adding element %.*s\n",element.returnType~" "~element.id);
				TreeIter iterChild = browserTreeStore.append(parentIter);
				char [][] subVals;
				subVals ~= element.getLongForm();
				subVals ~= RowType.ELEMENT;
				subVals ~= std.string.toString(element.getLineInCode());
				browserTreeStore.set(iterChild,cols,subVals);
			}
		}
		catch ( DirectoryError e)
		{
			printf("BrowserView addModuleElements error : %.*s\n",e.toString());
		}
	}

	private void addModuleEnums(TreeIter parentIter, Enum[] elements)
	{
		static int [3] cols = [0,1,2];
		if ( elements.length == 0 ) return;
		try
		{
			foreach(Enum element ; elements)
			{
			//printf("adding element %.*s\n",element.returnType~" "~element.id);
				TreeIter iterChild = browserTreeStore.append(parentIter);
				char [][] subVals;
				subVals ~= element.getLongForm();
				subVals ~= RowType.ELEMENT;
				subVals ~= std.string.toString(element.getLineInCode());
				browserTreeStore.set(iterChild,cols,subVals);
			}
		}
		catch ( DirectoryError e)
		{
			printf("BrowserView addModuleElements error : %.*s\n",e.toString());
		}
	}

	private void addModuleFunctions(TreeIter parentIter, Function[] elements)
	{
		static int [3] cols = [0,1,2];
		if ( elements.length == 0 ) return;
		try
		{
			foreach(Function element ; elements)
			{
			//printf("adding element %.*s\n",element.returnType~" "~element.id);
				TreeIter iterChild = browserTreeStore.append(parentIter);
				char [][] subVals;
				subVals ~= element.getLongForm();
				subVals ~= RowType.ELEMENT;
				subVals ~= std.string.toString(element.getLineInCode());
				browserTreeStore.set(iterChild,cols,subVals);
			}
		}
		catch ( DirectoryError e)
		{
			printf("BrowserView addModuleElements error : %.*s\n",e.toString());
		}
	}

	private void addModuleClasses(TreeIter parentIter, Class[] elements)
	{
		static int [3] cols = [0,1,2];
		if ( elements.length == 0 ) return;
		try
		{
			foreach(Class element ; elements)
			{
				TreeIter iterChild = browserTreeStore.append(parentIter);
				char [][] subVals;
				//printf("adding Class %.*s\n",element.getLongForm());
				subVals ~= element.getLongForm();
				subVals ~= RowType.ELEMENT;
				subVals ~= std.string.toString(element.getLineInCode());
				browserTreeStore.set(iterChild,cols,subVals);
				addModuleEnums(iterChild, element.enums);
				addModuleElements(iterChild, element.elements);
				addModuleFunctions(iterChild, element.functions);
			}
		}
		catch ( DirectoryError e)
		{
			printf("BrowserView addModuleElements error : %.*s\n",e.toString());
		}
	}


	/**
	 * Creates the simple toolbar for the project view.
	 * @return a Toolbar
	 */
	Toolbar getToolbar()
	{	
		Dispatcher dispatcher = Dispatcher.getDispatcher();
		Toolbar toolbar = new Toolbar();
		//toolbar.setIconSize(IconSize.SMALL_TOOLBAR);
		toolbar.setIconSize(IconSize.MENU);
		toolbar.setStyle(ToolbarStyle.ICONS);
		
		// this exist because of compiler
		// problems on "forward references to classes"
		void addButton(Stock stockID, char[] tooltip, char[] action)
		{
			Button button = (Button)toolbar.insertStock(stockID,tooltip,"",-1);
			dispatcher.addButtonClickedListener(this,button,action);
		}
			
		addButton(Stock.GO_UP,"Up","toolbar.up");
		addButton(Stock.GO_FORWARD,"Enter","toolbar.enter");
		addButton(Stock.OPEN,"Open","toolbar.open");
		
		return toolbar;
	}

	/**
	 * listens to user button clicks
	 * @param button
	 * @param action string
	 */
	void buttonClickedCallback(Button button, char [] action)
	{
		switch(action)
		{
	
			//case "toolbar.new": ; break;
			//case "toolbar.add": addFile(); break;
			//case "toolbar.remove": remove(); break;
				
			default:
				printf("Browser view - You press %.*s\n",action);
				break;
		}
	}

	char[] getCurrentDir()
	{
		return currentDir;
	}

	TreeView getTreeView()
	{
		return treeView;
	}

	/**
	 * A TreeStore for a Browser
	 */
	class BrowserTreeStore : TreeStore
	{
		this(GType[] columns)
		{
			super(columns);
		}
	}
	
	BrowserTreeView createBrowserTree()
	{
		browserTreeStore = new BrowserTreeStore(columns);
		//printf("projectTreeStore %X\n",projectTreeStore);
		BrowserTreeView treeView = new BrowserTreeView(browserTreeStore,this);
		TreeSelection ts = treeView.getSelection();
		ts.setMode(SelectionMode.MULTIPLE);
		treeView.setHeadersVisible(false);
		TreeViewColumn column = new TreeViewColumn("Elements",new CellRendererText(),"text", 0);
		treeView.appendColumn(column);

		return treeView;
	}

	/**
	 * A TreeView to display a browser structure
	 */
	class BrowserTreeView : TreeView , MouseButtonListener , MenuItemListener
	{
	
		Menu projectPopupMenu;
		Menu directoryPopupMenu;
		Menu filePopupMenu;
		
		BrowserView browserView;

		TreeStore treeStore;
		
		this(TreeStore treeStore, BrowserView browserView)
		{
			super(treeStore);
			this.treeStore = treeStore;
			this.browserView = browserView;
		//printf("treeStore %X\n",this.treeStore);
			Dispatcher d = Dispatcher.getDispatcher();
			directoryPopupMenu = buildDirectoryMenu();
			filePopupMenu = buildFileMenu();
			d.addMouseButtonListener(this,this);
			//d.addMenuItemListener(this,this);
		}

		Menu buildDirectoryMenu()
		{
			Menu menu = new Menu();
			MenuItem item;
			
			menu.append(new MenuItem(this,"_Filter","directoryPopupMenu.filter"));
			menu.append(new MenuItem(this,"_Hide","directoryPopupMenu.hide"));

			return menu;
		}
		
		Menu buildFileMenu()
		{
			Menu menu = new Menu();
			MenuItem item;
			
			menu.append(new MenuItem(this,"_Open","filePopupMenu.open"));
			menu.append(new MenuItem(this,"_Copy","filePopupMenu.copy"));
			menu.append(new MenuItem(this,"_Paste","filePopupMenu.paste"));
			menu.append(new MenuItem(this,"_Cut","filePopupMenu.cut"));
			menu.append(new MenuItem(this,"_Remove","filePopupMenu.remove"));

			return menu;
		}

		bit mouseButtonPressCallback(Widget widget, EventButton eventButton)
		{
			//printf("mouse pressed on project tree view with button %d and time %d\n",eventButton.button(),eventButton.getTime());
			if ( eventButton.button() == 3 )
			{
			}
			else
			{
			}
			return false;   // false : the event is passed to other listeners
		}
		
		bit mouseButtonReleaseCallback(Widget widget, EventButton eventButton)
		{
			//printf("mouse release on project tree view with button %d\n",eventButton.button());
			TreeIter iter = getSelectedIter();
			if ( iter !== null )
			{
				if ( eventButton.button() == 3 )
				{
					switch(iter.getValueString(1))
					{
						default:
							break;
					}
				}
				else if ( eventButton.button() == 2 )
				{
					//printf("tree mouse button == 2\n");
					//printf("tree mouse iter 0 == %.*s\n",iter.getValueString(0));
					//printf("tree mouse iter 1 == %.*s\n",iter.getValueString(1));
					//printf("tree mouse iter 2 == %.*s\n",iter.getValueString(2));
					switch(iter.getValueString(1))
					{
						case RowType.ELEMENT:
							try
							{
								int line = std.conv.toInt(iter.getValueString(2));
								browserView.workspace.gotoLine(line);
							}
							catch
							{
								printf("error converting to int\n");
							}
							break;
						
						default:
							break;
					}
				}
				else
				{
					//projectMenu.popup(eventButton.button(),eventButton.getTime());
				}
			}
			return false;   // false : the event is passed to other listeners
		}

		/**
		 * Menu listener callback
		 */
		void activateCallback(MenuItem menuItem, char [] action)
		{
			TreeIter[] iters = getSelectedIters();
			switch(action)
			{
				
				case "filePopupMenu.open":
					foreach ( TreeIter iter ; iters )
					{
						if ( iter !== null && "F"==iter.getValueString(1) )
						{
							browserView.workspace.openFile(iter.getValueString(2));
						}
					}
					break;
					
				default:
					printf("Menu option %.*s\n", action);
					break;
			}
		}
		
		void activateItemCallback(MenuItem menuItem, char [] action)
		{
			switch(action)
			{
				default:
					printf("Menu option %.*s\n", action);
					break;
			}
		}
		
	}
	
}
