/*
 * This file is part of leds.
 * 
 * leds is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * leds is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with leds; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

module leds.DSpell;

private import dui.All;

private import leds.Workspace;
private import leds.POpen;

private import std.string;

public
class DSpell : Window , ButtonClickedListener , POpenLineListener
{

	private:	

	int left = -1;
	int top = -1;

	char[] testStr = "";
	char[] replaceStr;
	char[][] replaceStrList;

	Label testStrLabel;
	Entry replaceEntry;
	DUIList wordList;

	Workspace workspace;

	public:
	this(Workspace workspace)
	{
		super("Spell for leds");
		this.workspace = workspace;
		setBorderWidth(7);
		setup();
		setTransientFor(workspace);
	}
	
	void show(char[] testStr)
	{
		setupWordList(testStr);
		this.testStr = testStr;
		testStrLabel.setText("Spell: " ~ testStr);
		if (left>=0 )
		{
			move(left, top);
		}
		super.show();
	}
	
	int setupWordList(char[] testStr)
	{
		wordList.clear();
		POpen popen = new POpen(this,"echo " ~ testStr ~ " | ispell",POpenMode.READ,false);
		return 1;
	}

	bit popenLineCallback(char[] line)
	{
		int pos = std.string.find(line,"how about:");
		if ( pos >= 0 )
		{
			char[][] tokens = std.string.split(line[pos+"how about:".length..line.length],",");
			foreach(char[] token; tokens)
			{
				int p = 0;
				while( (p<token.length) && (token[p] == ' ') )
				{
					p++;
				}
				
				wordList.append(token[p..token.length]);
			}
		}
		return false;
	}
	void popenClosed(char[] command)
	{
	}
	

	void hide()
	{
		getPosition(&left, &top);
		super.hide();
	}
	
	private void setup()
	{
		VBox vbox = new VBox(false,4);		testStrLabel = new Label("");

		
		testStrLabel = new Label("");
		replaceEntry = new Entry("");
		wordList = new DUIList();
		
		Dispatcher dispatcher = Dispatcher.getDispatcher();
		
		Table table = new Table(2,3,false);
		
		table.attach(testStrLabel,0,1,0,1,AttachOptions.FILL,AttachOptions.FILL,2,2);
		table.attach(replaceEntry,0,1,1,2,AttachOptions.FILL,AttachOptions.FILL,2,2);
		table.attach(wordList,0,1,2,5,AttachOptions.FILL,AttachOptions.FILL,2,2);

		Button button = new Button("_Replace");
		table.attach(button,1,2,2,3,AttachOptions.FILL,AttachOptions.FILL,2,2);
		dispatcher.addButtonClickedListener(this,button,"replace");

		button = new Button("Replace _All");
		table.attach(button,2,3,2,3,AttachOptions.FILL,AttachOptions.FILL,2,2);
		dispatcher.addButtonClickedListener(this,button,"replaceAll");

		button = new Button("_Ignore");
		table.attach(button,1,2,3,4,AttachOptions.FILL,AttachOptions.FILL,2,2);
		dispatcher.addButtonClickedListener(this,button,"ignore");

		button = new Button("_Cancel");
		table.attach(button,2,3,3,4,AttachOptions.FILL,AttachOptions.FILL,2,2);
		dispatcher.addButtonClickedListener(this,button,"cancel");

		Alignment alignment = new Alignment(1,0.5,1,1);
		alignment.add(table);
		vbox.packStart(alignment,false,true,0);
		
		add(vbox);
	}
	
	
	// interface KeyListener

	bit keyPressCallback(Widget widget, EventKey event)
	{
		printf("Key pressed " ~ std.string.toString(event.getKeyVal()) ~ \n);
		workspace.getSystemConsole().append("Key pressed " ~ std.string.toString(event.getKeyVal()) ~ \n);
		return false;
	}
	bit keyReleaseCallback(Widget widget, EventKey event)
	{
		return false;
	}

	
	void buttonClickedCallback(Button button, char[] action)
	{
		bit cancel = false;
		bit replace = false;
		bit find = false;
		switch ( action )
		{
			case "replace":
				replace = true;
				break;

			case "replaceAll":
				replace = true;
				break;
			
			case "ignore":
				break;
				
			case "cancel": cancel = true; break;
			
			default:
				printf("find window action = %.*s\n",action);
				break;
		}
		
		hide();

		//findStr = findCombo.getSelectedText();
		replaceStr = replaceEntry.getText();


		if ( !cancel )
		{
			workspace.findReplace(testStr, replaceStr, replace,find,0, false);
		}
	}
	

	/**
	 * DUIList
	 */
	class DUIList : ScrolledWindow
	{
		TreeView treeView;
		ListStore model;
			
		this()
		{
			super(PolicyType.AUTOMATIC, PolicyType.AUTOMATIC);
			setShadow(ShadowType.ETCHED_IN);
			
			model = getModel();
			
			treeView = new TreeView(model); // add model here

			treeView.setRulesHint(true);
			treeView.getSelection().setMode(SelectionMode.SINGLE);
		
			addColumns(treeView, "select");
			add(treeView);
			treeView.setSizeRequest(150,150);		

		}
		
		ListStore getModel()
		{

			if ( model === null )
			{
				int i = 0;
				DUIType []columns;
		
				columns ~= DUIType.STRING;
		
				model = new ListStore(columns);
				
			}
			
			return model;
		}
		
		void addColumns(TreeView treeView, char[] heading)
		{
			//Dispatcher dispatcher = Dispatcher.getDispatcher();
			CellRenderer renderer;
			TreeModel model = treeView.getModel();
			renderer = new CellRendererText();
			int pos = treeView.insertEditableColumn(0, heading, renderer, true);
			//TreeViewColumn column = new TreeViewColumn("Number",new CellRendererText(),"text", 0);
			//treeView.appendColumn(column);
			
			TreeViewColumn column = treeView.getColumn(pos-1);
			column.setResizable(true);
			column.setSortColumnID(0);
			column.setSortIndicator(true);
			//dispatcher.addCellListener(this,renderer,model,pos-1);
	
		}

		void append(char[][] values)
		{
			foreach(char[] value ; values)
			{
				append(value);
			}
		}
		
		void append(char[] value)
		{
			TreeIter iter = new TreeIter();
			model.append(iter);
			model.setValue(iter,0,value);
		}
		
		void clear()
		{
			getModel().clear();
		}
	}
}
