/*
 * This file is part of leds.
 * 
 * leds is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * leds is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with leds; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

module leds.DockerTools;


private import dui.All;
private import leds.Docker;
private import leds.SystemConsole;

/**
 * A place to dock widget.
 * for now it's implemented through a notebook
 */
public:
class DockerTools : Docker , ButtonClickedListener
{

	HBox hbox;
	Toolbar toolbar;
	
	this(char[] name)
	{
		this(name,false);
	}
	
	this(char[] name, bit horizontal)
	{
		super(name, horizontal);
		
		setupBox();
	}

	Widget getWidget()
	{
		return hbox;
	}
	
	void setupBox()
	{
		hbox = new HBox(false,0);
		toolbar = new Toolbar();
		toolbar.setOrientation(Orientation.VERTICAL);
		Dispatcher dispatcher = Dispatcher.getDispatcher();
		;
		//toolbar.setIconSize(IconSize.SMALL_TOOLBAR);
		toolbar.setIconSize(IconSize.MENU);
		toolbar.setStyle(ToolbarStyle.ICONS);
		
		void addButton(Stock stockID, char[] tooltip, char[] action)
		{
			Button button = (Button)toolbar.insertStock(stockID,tooltip,"",-1);
			dispatcher.addButtonClickedListener(this,button,action);
		}
			
		//addButton(Stock.GO_UP,"Up","toolbar.up");
		//addButton(Stock.GO_FORWARD,"Enter","toolbar.enter");
		addButton(Stock.CLOSE,"Close","toolbar.close");
		addButton(Stock.CLEAR,"Clear","toolbar.clear");
		addButton(Stock.GOTO_TOP,"Top","toolbar.toggleTop");
		
		hbox.packStart(toolbar,false,false,0);
		hbox.packStart(notebook,true,true,0);

	}

	/**
	 * listens to user button clicks
	 * @param button
	 * @param action string
	 */
	void buttonClickedCallback(Button button, char [] action)
	{
		switch(action)
		{
	
			case "toolbar.clear":
				if ( getCurrentDockable() )
				{
					getCurrentDockable().clear();
				}
				break;

			case "toolbar.close":
				undockCurrent();
				break;
			//case "toolbar.new": ; break;
			//case "toolbar.add": addFile(); break;
			//case "toolbar.remove": remove(); break;
				
			default:
				printf("FS view - You press %.*s\n",action);
				break;
		}
	}

}
