/*
 * This file is part of leds.
 * 
 * leds is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * leds is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with leds; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

module leds.GotoLineDialog;

import dui.All;
import leds.Workspace;
import leds.Scintilla;

import std.string;

/**
 * A window enter the line number to go to .
 */
public:
class GotoLineDialog : Window , ButtonClickedListener
{

	private:	
	SpinButton spin;
	
	Workspace workspace;

	int left = -1;
	int top = -1;

	int maxLines = 0;
	Label rangeLabel;
	
	public:
	this(Workspace workspace)
	{
		super("Go to line");
		this.workspace = workspace;
		setBorderWidth(7);
		setup();
		setTransientFor(workspace);
	}
	
	void show(int maxLines)
	{
		this.maxLines = maxLines;
		spin.setAdjustment(new Adjustment(spin.getValue(), 1, maxLines, 1, 10, 10));
		rangeLabel.setText("goto line [1,"~std.string.toString(maxLines)~"]");
		show();
	}
	
	private void show()
	{
		if (left>=0 )
		{
			move(left, top);
		}
		super.show();
	}
	
	void hide()
	{
		getPosition(&left, &top);
		super.hide();
	}
	
	private void setup()
	{
		VBox vbox = new VBox(false,4);
		
		spin = new SpinButton(new Adjustment(1, 1, maxLines, 1, 10, 10),1,0);
		
		rangeLabel = new Label("goto line [1,"~std.string.toString(maxLines)~"]");
		
		vbox.packStart(rangeLabel,false,false,0);
		vbox.packStart(spin,false,false,0);

		Dispatcher dispatcher = Dispatcher.getDispatcher();
	

		Table table = new Table(1,2,true);
		
		Button button = new Button("_Goto");
		table.attach(button,0,1,0,1,AttachOptions.EXPAND,AttachOptions.EXPAND,2,2);
		dispatcher.addButtonClickedListener(this,button,"goto");

		button = new Button("_Cancel");
		table.attach(button,1,2,0,1,AttachOptions.EXPAND,AttachOptions.EXPAND,2,2);
		dispatcher.addButtonClickedListener(this,button,"cancel");

		vbox.packStart(table,false,false,0);
		
		add(vbox);
	}
	
	void buttonClickedCallback(Button button, char[] action)
	{
		bit cancel = false;
		switch ( action )
		{
			case "cancel": cancel = true; break;
			
			default:
				printf("find window action = %.*s\n",action);
				break;
		}
		
		hide();
		
		if ( !cancel )
		{
			workspace.gotoLine(spin.getValueAsInt);
		}
		
	}

}
