/*
 * This file is part of leds.
 * 
 * leds is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * leds is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with leds; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

module leds.ImageView;

private import dui.All;

private import leds.Workspace;
private import leds.FileView;

private import std.path;
private alias std.path filePath;

/**
 * A scrolled window that contains a scintilla widget
 */
public:
class ImageView : FileView
{

	Image image;
	
	public:
	
	/**
	 * creates a new empty CodeView
	 */
	this(Workspace workspace)
	{
		this(workspace, null);
	}

	/**
	 * creates a new CodeView and reads in a file
	 * @param fileName the fileName
	 */
	this(Workspace workspace, char[] fileName)
	{
		super(workspace, fileName);
		loadFile(fileName);
	}
	
	static bit isImageFile(char[] fullName)
	{
		char[] ext = filePath.getExt(fullName);
		return ext == "png"
				|| ext == "jpg"
				|| ext == "bmp"
				|| ext == "xpm"
				|| ext == "gif"
				;
	}
	
	/**
	 * loads a file into the view
	 * @param fileName the file to load
	 * @return true if sucesseful
	 */
	bit loadFile(char[] fileName)
	{
		image = new Image(fileName);
		addWithViewport(image);
		return true;
	}

	/**
	 * Indicates if the text was modified
	 * @return true if the text was modified
	 */
	bit getModified(){return false;}
	
	/**
	 * indicates if this buffer is a empty never used buffer
	 * @return true if this instance should be loaded with a new file
	 */
	bit getEmpty(){return false;}
	 
	/**
	 * checks if the buffer needs to be saved
	 * @return true if the buffer can be closed
	 */
	bit allowClose(){return true;}
	
	/**
	 * saves the current file into the hard drive
	 * @return true if sucesseful
	 */
	bit saveFile(){return true;}
	
	/**
	 * renames and saves the current file into the hard drive
	 * @return true if sucesseful
	 */
	bit saveFile(char[] fileName){return true;}
	
	/**
	 * renames our current file name
	 */
	void rename(char[] fileName){}

	/**
	 * execute an action if action is known.
	 * @param action
	 * @return true if the action was processed (even if the execution failled)
	 */
	public bit execAction(char[] action){return false;}

	/**
	 * find for the text on the toolbar find text entry
	 */
	void findText(){}
	
	/**
	 * goto a line
	 */
	void gotoLine(int lineNumber){}
	
	/**
	 * Finds or replaces text
	 * @param findStr the string to find
	 * @param replaceStr the string to replace, the new string
	 * @param replace if true the replaceStr should replace the selected text
	 * @param find if true after replace do another find
	 * @param flags the find mode @see Scintilla
	 */
	void findReplace(char[] findStr, char[] replaceStr, bit replace, bit find,int flags, bit backwards){}

}
