/*
 * This file is part of leds.
 * 
 * leds is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * leds is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with leds; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

module leds.SystemConsole;

private import dui.All;

private import leds.Workspace;
private import leds.FileView;
private import leds.Docker;
private import leds.POpen;


/**
 * SystemConsole
 */
public:
class SystemConsole : ScrolledWindow , Dockable , POpenLineListener , MenuItemListener
{
	
	TextView textView;
	Workspace workspace;
	Menu tabMenu;
	Docker docker;
	
	bit executing;
	
	/**
	 * Creates a new System console for a workspace
	 * @param workspace
	 */
	this(Workspace workspace)
	{
		this.workspace = workspace;
		setPolicy(PolicyType.AUTOMATIC, PolicyType.AUTOMATIC);
		textView = new TextView();
		add(textView);
	}

	Widget getWidget()
	{
		return this;
	}
	
	char[] getTabName()
	{
		return "output";	// change name when we can type in command
	}

	Menu getTabMenu()
	{
		if ( tabMenu === null )
		{
			tabMenu = new Menu();
			MenuItem item;
			
			tabMenu.append(new MenuItem(this,"_Float","tabMenu.float"));
		}
		return null;
	}

	void setDocker(Docker docker)
	{
		this.docker = docker;
	}

	/**
	 * Gets this dockable file view
	 * @return null because with don't have a file view
	 */
	FileView getFileView()
	{
		return null;
	}
	
	/**
	 * detrimines if a close button shoud be visible
	 */
	bit allowCloseButton()
	{
		return false;
	}

	bit allowClose()
	{
		return false;
	}
	
	/**
	 * clears all previews output
	 */
	bit clear()
	{
		textView.getBuffer().setText("");
		return true;
	}
	
	void print(char[] text)
	{
		textView.insertText(text);
	}

	void append(char[] text)
	{
		textView.appendText(text);
	}

	/**
	 * executes a system command
	 * @param command
	 */
	void exec(char[] command)
	{
		append("\n*** Executing: "~command~"\n");
		executing = true;
		POpen popen = new POpen(this, command ~ " 2>&1",POpenMode.READ, true);
	}

	//bit popenByteCallback(byte byteRead)
	//{
	//	printf("%c ",byteRead, byteRead);
	//	return false;
	//}
	
	bit popenLineCallback(char[] line)
	{
		//TextBuffer buf = textView.getBuffer();
		//TextIter iter = new TextIter();
		append(line~"\n");
		//gdouble within_margin = 0.0;
		//bit use_align = false;
		//gdouble xalign = 0.0;
		//gdouble yalign = 0.0;
		//textView.scrollToMark(buf.createMark(null,iter,true), within_margin, use_align, xalign, yalign);
		return false;
	}
	
	void popenClosed(char[] command)
	{
		//TextBuffer buf = textView.getBuffer();
		//TextIter iter = new TextIter();
		//buf.getEndIter(iter);
		append("\n*** Finished: "~command~"\n");
		//gdouble within_margin = 0.0;
		//bit use_align = false;
		//gdouble xalign = 0.0;
		//gdouble yalign = 0.0;
		//textView.scrollToMark(buf.createMark(null,iter,true), within_margin, use_align, xalign, yalign);
		executing = false;
	}
	
	void activateItemCallback(MenuItem menuItem, char [] action)
	{
		printf("Workspace.MenuListener.ctivateItemCallback %.*s\n",action);
	}

	/**
	 * Menu listener callback
	 */
	void activateCallback(MenuItem menuItem, char [] action)
	{
		switch(action)
		{
			case "tabMenu.float":
				break;

			default:
				break;
		}
	}

};
